;**********************************************************
;*  fc003.asm - measures the period on the signal clocking TMR1
;**********************************************************
;*  It uses a count of cycles over a period.
;*  It uses the 12F683 CCP to measure the period.
;* It samples TMR1 with CCP, then again and outputs the difference.
;*
;*  based on FC002.asm which configures a PIC 12F675 or 12F683 to measure frequency.
;*
;* TMR1 is clocked by 4Hz internal oscillator, 
;* TMR1 clocks at 1MHz. 
;* really needs to be 10Mhz external input.
;*
;* Example output
;
#ifdef comment
t(0,0x9CFD,00040189,0)
t(0,0x9CAB,00040107,0)
t(0,0x9CCC,00040140,0)
t(0,0x9CD9,00040153,0)
t(0,0x9CD8,00040152,0)
t(0,0x9C7F,00040063,0)
t(0,0x9CE6,00040166,0)
t(0,0x9CB7,00040119,0)
t(0,0x9C9A,00040090,0)
t(0,0x9CBB,00040123,0)
t(0,0x9C7D,00040061,0)
t(0,0x9CBD,00040125,0)
t(0,0x9CF1,00040177,0)
t(0,0x9CB8,00040120,0)
t(0,0x9C95,00040085,0)
t(0,0x9C70,00040048,0)
t(0,0x9C42,00040002,0)
t(0,0x9C71,00040049,0)
t(0,0x9CA9,00040105,0)
t(0,0x9CCE,00040142,0)
t(0,0x9D0E,00040206,0)
t(0,0x9CE0,00040160,0)
#endif
;*
;*  Copyright 2005,2016 Douglas Rice
;*
;*
;**********************************************************

;--------------------------------------------------------------------------
; Sec 1.0	Build support for 12F675 or 12F683
;-------------------------------------------------------------------------
; uncomment one of these below or define it on the command line
;#define using12F675
#define using12F683


#ifdef using12F675
;
; 12F675 not supported as it does not has a CCP
;
;

	list p=12f675
	; Include file, change directory if needed
	include "p12f675.inc"
	__idlocs  0xF675	
; Using internal 4Mhz Osc	
;	__config  _PWRTE_ON & _WDT_OFF & _BODEN_OFF & _MCLRE_OFF & _INTRC_OSC_NOCLKOUT
;	__config  _PWRTE_ON & _WDT_OFF & _BODEN_OFF & _MCLRE_OFF & _EC_OSC
	__config  _PWRTE_ON & _WDT_OFF & _BODEN_OFF & _MCLRE_OFF & _INTRC_OSC_CLKOUT

; Using external 4Mhz Osc	
;	__config  _PWRTE_ON & _WDT_OFF & _BODEN_OFF & _MCLRE_OFF & _XT_OSC
#endif


#ifdef using12F683
	list p=12f683
	; Include file, change directory if needed
	include "p12f683.inc"
	__idlocs  0xF683	

; NOTE:- 12F683 has failsafe clock so can run off internal clock or external clock.
; MORE WORK IS REQUIRED TO SET THIS UP.
; Need to set up OSCON to select external oscillator. This is to be done.
; Using internal 4Mhz Osc if FCMEM_ is not diabled explicitly.	
;
;_FCMEN_ON                    EQU     H'3FFF'
;_FCMEN_OFF                   EQU     H'37FF'

;	__config  _PWRTE_ON & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _INTRC_OSC_NOCLKOUT
;	__config  _PWRTE_ON & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _INTRC_OSC_CLKOUT

; External clock with 
	__config  _PWRTE_ON & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _EC_OSC & _FCMEN_ON & _IESO_ON
;	__config  _PWRTE_ON & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _XT_OSC & _FCMEN_ON & _IESO_ON


; Using external 4Mhz Osc	
;	__config  _PWRTE_ON & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _XT_OSC

;
; 12F683 has extra CMCON1 for TMR1 sync
;
#define	CMCON CMCON0
 
#endif

;--------------------------------------------------------------------------
; Sec 1.2	Variables
;----1---------------------------------------------------------------------

	cblock 	0x20
	  ; variables used for context saving 
	  w_temp
	  status_temp

	  SRtxTemp
	  SRtxCnt
	  SRout
	  SRdel


	  cnt1
	  cnt2
	  cnt3

	  tmpCcp1L
	  tmpCcp1H

	  lastCcp1L
	  lastCcp1H

	  diffCcp1L
	  diffCcp1H
	  		
	endc
	
	cblock		; variables used in BCD.INC
	BCDip0	
	BCDip1	
	BCDip2	
	BCDip3
	
	BCDop0                    ; RAM Assignments
	BCDop1      
	BCDop2      
	BCDop3      
	BCDop4      

	BCDcount  
	BCDtemp   
	endc

;--------------------------------------------------------------------------
; Sec 1.4	MACRO for tables straddling memory page
;-------------------------------------------------------------------------

TEST_STRADDLE	MACRO	START
	if high( $ ) != high( START ) 
	    Error "Table straddles Page Boundary "
	endif
	endm

	; Start at the reset vector
	org	0x000
	goto	Init



;--------------------------------------------------------------------------
; Sec 2.0	Code
;-------------------------------------------------------------------------

Init

	org	0x010

#ifdef using12F675	
;	banksel 0x3FF
;	CALL 	0x3FF
	MOVWF	OSCCAL
#endif
#ifdef using12F683
	nop
;TBD
#endif


	banksel	TRISIO 

;	BSF		STATUS, RP0
	movlw	0xDF
	MOVWF	TRISIO		; Set output high to start Measurement
	BCF		STATUS, RP0

;
;
;
;



	; Configure the Comparator to low power
	; for 12F683 #define	CMCON CMCON0
	banksel  CMCON
	movlw	B'00010111'	;
	movwf	CMCON

	; turn off A to D converter
	banksel  ADCON0
	movlw	B'00000000'	;
	movwf	ADCON0

	banksel  ANSEL
	movlw	B'00000000'	;Fosc/8, A/D disabled
	movwf	ANSEL

	; Set up the comparator and Vref for the freq input	

	banksel  VRCON
	movlw	1 << VREN | 0<<VRR | 5	; level = Vdd * 4 / 24 = 1.0 Volts if Vdd is 5 volts.
	movwf	VRCON
;;;;;;;;;;;;;;;;;;;

InitCCP
;
; set up TMR1 and CCP
;

;

	banksel	T1CON

	movlw	0 << T1GINV | 0 << T1GE  | 0 << T1CKPS0 | 0 << T1OSCEN | 0 << NOT_T1SYNC | 0 << TMR1CS | 0 << TMR1ON
    movwf	T1CON
	movlw	0 << T1GINV | 0 << T1GE  | 0 << T1CKPS0 | 0 << T1OSCEN | 0 << NOT_T1SYNC | 0 << TMR1CS | 1 << TMR1ON
    movwf	T1CON

;
; set up CCP to capture Signal and capture Period.
;

;
; If measuring 50Hz, period os 20ms which fits in the 16bit 
;

    banksel	CCP1CON
	movlw	0<<DC1B0 | B'0100' << CCP1M0 
    movwf	CCP1CON
	banksel	GPIO


StartCCP

	movlw	' '
	call	SRtxChar


	movlw	0x0D
	call	SRtxChar

	movlw	0x0A
	call	SRtxChar

	movlw	't'
	call	SRtxChar
	movlw	'('
	call	SRtxChar

	movlw	'0'
	call	SRtxChar
	movlw	','
	call	SRtxChar


waitCCP
	; wait for the CCP
	banksel PIR1
	btfss   PIR1,CCP1IF
	goto	waitCCP

;	Clear TMR1 - but jitter will be introduced by software.
;	clrf	TMR1L
;	clrf	TMR1H

	nop
	nop
	bcf		PIR1,CCP1IF

;copy last reading
	banksel	tmpCcp1H
	movfw	tmpCcp1H
	movwf	lastCcp1H
	movfw	tmpCcp1L
	movwf	lastCcp1L

	banksel	CCPR1H
	movfw	CCPR1H
	movwf	lastCcp1H
	banksel	CCPR1H
	movfw	CCPR1L
	movwf	lastCcp1L


	; wait
waitCCP2
	banksel PIR1
	btfss   PIR1,CCP1IF
	goto	waitCCP2
	bcf		PIR1,CCP1IF

;
	banksel	CCPR1H
	movfw	CCPR1H
	movwf	tmpCcp1H
	banksel	CCPR1H
	movfw	CCPR1L
	movwf	tmpCcp1L

	;
	; Subtract new measurement from previous measurement
	;
	movfw	lastCcp1L
	subwf	tmpCcp1L,w
	movwf	diffCcp1L

	; borrow is needed
	skpc
	incf	lastCcp1H,f

	movfw	lastCcp1H
	subwf	tmpCcp1H,w
	movwf	diffCcp1H

	; 

	banksel  GPIO

#ifdef not
	call	delay
	movlw	'0'
	call	SRtxChar
	movlw	'x'
	call	SRtxChar

	banksel	GPIO
	swapf	tmpCcp1H,W		;
	call 	SRwrtHexNibble
	movf	tmpCcp1H,W		;
	call 	SRwrtHexNibble
	swapf	tmpCcp1L,W		;
	call 	SRwrtHexNibble
	movf	tmpCcp1L,W		;
	call 	SRwrtHexNibble
	movlw	','
	call	SRtxChar

	movlw	'0'
	call	SRtxChar
	movlw	'x'
	call	SRtxChar

	banksel	GPIO
	swapf	lastCcp1H,W		;
	call 	SRwrtHexNibble
	movf	lastCcp1H,W		;
	call 	SRwrtHexNibble
	swapf	lastCcp1L,W		;
	call 	SRwrtHexNibble
	movf	lastCcp1L,W		;
	call 	SRwrtHexNibble

#endif
	movlw	'0'
	call	SRtxChar
	movlw	'x'
	call	SRtxChar

	banksel	GPIO
	swapf	diffCcp1H,W		;
	call 	SRwrtHexNibble
	movf	diffCcp1H,W		;
	call 	SRwrtHexNibble
	swapf	diffCcp1L,W		;
	call 	SRwrtHexNibble
	movf	diffCcp1L,W		;
	call 	SRwrtHexNibble

	movlw	','
	call	SRtxChar





	;
	; Copy result for binary to decimal
	;

;	clrf	BCDip0
;	clrf	BCDip1

	movf	diffCcp1L,W		;
	movwf	BCDip0

	movf	diffCcp1H,W		;
	movwf	BCDip1

	clrf	BCDip2
	clrf	BCDip3

	call    B2_BCD          ; After conversion the Decimal Number

	; now set up the display for the display.

	SWAPF	BCDop3,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop3
	CALL	SRwrtHexNibble

	SWAPF	BCDop2,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop2
	CALL	SRwrtHexNibble

	SWAPF	BCDop1,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop1
	CALL	SRwrtHexNibble

	SWAPF	BCDop0,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop0
	CALL	SRwrtHexNibble


	movlw	','
	call	SRtxChar

; Print out OSCON to show what Oscillator is being used
	movlw	'0'
	call	SRtxChar
	movlw	'x'
	call	SRtxChar

	banksel OSCCON
	swapf	OSCCON,w
	CALL	SRwrtHexNibble

	banksel OSCCON
	movfw	OSCCON
	CALL	SRwrtHexNibble
	movlw	','
	call	SRtxChar

	movlw	'0'
	call	SRtxChar

	movlw	')'
	call	SRtxChar

	banksel OSCCON
	movlw	0x60 ; 6 << IRCF0 | 0 << SCS
	movwf	OSCCON

	goto	StartCCP


;--------------------------------------------------------------------------
; Sec 2.1	Serial code - originally for PIC16F84 clocked at 32kHz
;-------------------------------------------------------------------------


SRwrtHexNibble	; currently only displays 0..9, A..F
	banksel	SRtxTemp
	ANDLW	0x0F
	ADDLW	0x06	; is it A..F, if so trigger a digit overflow
	SKPNDC
	ADDLW	7; subtract 10, then add 'A'-'0'
	ADDLW	0x30-6	; Subtract extra 6 added to cause DC
	MOVWF	SRout
	GOTO	SRtxChar



SRtxChar
	;
	;
	;Output Start Bit
	;
	;Output LSB first
    ;
    ; -/=/=/=/=/=/=/=/=/=/__
	; START/lsb...msb/stop
	;
	; The PIC pin is directly connected to PIN2 of the RS232 socket
    ; Normally a MAX232 would invert the signal
    ; so Start is 1, the bits are sent LSB first and the stop is 0
    ;
    ;http://www.lookrs232.com/rs232/waveforms.htm
    ;
    ;


TxOnBit  equ 0
	
	banksel	SRtxTemp
	movwf	SRtxTemp
	comf	SRtxTemp,f

	
	BCF		GPIO,TxOnBit
	MOVLW	~( 1 << TxOnBit ) 
	
;	TRIS	GPIO
;	BSF		STATUS, RP0
	banksel TRISIO
	MOVWF	TRISIO		; Set output high to start Measurement
;	BCF		STATUS, RP0

	banksel GPIO
	call	delay	

	; 1200 baud output
	; move the char into SRtxtemp, it is destroyed.
	;Start Bit
	BSF		GPIO,TxOnBit
	GOTO	$+1
	call	delay	
	movlw	4
	movwf	SRtxCnt
	GOTO 	$+1
SRtxCharLp1

	; This outputs two bits at a time 
	; this takes 6.5 cycles each when using osc =32768Hz and 1200 baud.
	
	; After start bit, which is 1, then test each bit
	; I need to set PA:0 to same as LSB SRtxTemp
	; this compares PA:0 with SRtxTemp so see if it needs toggling
	; The xorwf PORTA,f causes the bit to be toggled if required.

	; Toggle bit if next bit is different 
	RRF		SRtxTemp,f
	movfw	STATUS
	andlw	1 << C		; mask off bit
	addlw   ( 1 << TxOnBit) -1	; shift bit by using a ripple carry
		
	xorwf	GPIO,W
	andlw	0x1 << TxOnBit
	xorwf	GPIO,f		;t+6,19

	GOTO	$+1	
	call	delay	
	RRF		SRtxTemp,f

	movfw	STATUS
	andlw	1 << C
	addlw   ( 1 << TxOnBit) -1

	xorwf	GPIO,W
	andlw	0x1 << TxOnBit
	xorwf	GPIO,f		;t+11,24
	GOTO	$+1	
	call	delay	
	decfsz	SRtxCnt
	goto	SRtxCharLp1
	GOTO	$+1	
	GOTO	$+1	
	; output stop bit
	BCF		GPIO,TxOnBit
	call	delay	
	GOTO	$+1	
	GOTO	$+1	

	; add some more stop bits
	BCF		GPIO,TxOnBit
	call	delay	
	GOTO	$+1	
	GOTO	$+1	

	BCF		GPIO,TxOnBit
	call	delay	
	GOTO	$+1	
	GOTO	$+1	
	RETURN


delay	
	; one bit delay
	nop
;	movlw	0x03  ; delay for 32K

	movlw	0xCD  ; delay for 1200K
	movwf	SRdel
delay1	
	NOP
	decfsz	SRdel
	goto	delay1
	return


;--------------------------------------------------------------------------
; Sec 5.1	Binary to BCD
;-----------------------------------------------------------------

#DEFINE	buildbcdBinDec24
;	BCF	PORTA,0

;	call	bcdBinDec24
	banksel	BCDop1
	call    B2_BCD          ; After conversion the Decimal Number

CNdispCount
	; now set up the display for the display.

	SWAPF	BCDop3,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop3
	CALL	SRwrtHexNibble

	SWAPF	BCDop2,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop2
	CALL	SRwrtHexNibble

	SWAPF	BCDop1,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop1
	CALL	SRwrtHexNibble

	SWAPF	BCDop0,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop0
	CALL	SRwrtHexNibble


	; we have displayed it so prevent another display
	return


;--------------------------------------------------------------------------
; Sec 5.2	Binary to BCD -Procedure
;-------------------------------------------------------------------------
	include	"bcd.inc"



	end


